/*
  Copyright (C) 2008  Hideki EIRAKU
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
*/

#include <windows.h>
#include <windowsx.h>
#include "icon.h"
#include "options.h"

extern HINSTANCE hinst;

struct iconinfo
{
  int x, y;
  int bitspixel;
  int planes;
  HDC dc[2];
  HBITMAP bitmap[2];
  HBRUSH brush[2][2];
};

static struct iconinfo icon;

static void
freebrush (void)
{
  int i, j;

  for (i = 0; i < 2; i++)
    for (j = 0; j < 2; j++)
      if (icon.brush[i][j])
	{
	  DeleteBrush (icon.brush[i][j]);
	  icon.brush[i][j] = NULL;
	}
}

static void
freeobjects (void)
{
  int i;

  for (i = 0; i < 2; i++)
    {
      if (icon.dc[i])
	{
	  DeleteDC (icon.dc[i]);
	  icon.dc[i] = NULL;
	}
      if (icon.bitmap[i])
	{
	  DeleteBitmap (icon.bitmap[i]);
	  icon.bitmap[i] = NULL;
	}
    }
  freebrush ();
}

void
icon_update (void)
{
  COLORREF c[2];

  freebrush ();
  if (opt.fgcolortype & 2)
    c[0] = RGB (255, 255, 255);
  else
    c[0] = RGB (0, 0, 0);
  if (opt.bgcolortype & 2)
    c[1] = RGB (255, 255, 255);
  else
    c[1] = RGB (0, 0, 0);
  icon.brush[0][0] = CreateSolidBrush (c[0]);
  icon.brush[0][1] = CreateSolidBrush (c[1]);
  icon.brush[1][0] = CreateSolidBrush (opt.fgcolor);
  icon.brush[1][1] = CreateSolidBrush (opt.bgcolor);
}

static void
createobjects (HDC hdc)
{
  RECT rect;

  icon.bitmap[0] = CreateBitmap (icon.x, icon.y, 1, 1, NULL);
  icon.bitmap[1] = CreateBitmap (icon.x, icon.y, icon.planes, icon.bitspixel,
				 NULL);
  icon.dc[0] = CreateCompatibleDC (hdc);
  icon.dc[1] = CreateCompatibleDC (hdc);
  (void) SelectBitmap (icon.dc[0], icon.bitmap[0]);
  (void) SelectBitmap (icon.dc[1], icon.bitmap[1]);
  icon.brush[0][0] = NULL;
  icon.brush[0][1] = NULL;
  icon.brush[1][0] = NULL;
  icon.brush[1][1] = NULL;
  icon_update ();
  rect.left = 0;
  rect.top = 0;
  rect.right = icon.x;
  rect.bottom = icon.y;
  FillRect (icon.dc[0], &rect, icon.brush[0][1]);
  FillRect (icon.dc[1], &rect, icon.brush[1][1]);
}

static void
prepare (HWND hwnd)
{
  HDC hdc;
  struct iconinfo ii;

  hdc = GetDC (hwnd);
  ii.x = GetSystemMetrics (SM_CXSMICON);
  ii.y = GetSystemMetrics (SM_CYSMICON);
  ii.bitspixel = GetDeviceCaps (hdc, BITSPIXEL);
  ii.planes = GetDeviceCaps (hdc, PLANES);
  if (icon.x != ii.x || icon.y != ii.y || icon.bitspixel != ii.bitspixel ||
      icon.planes != ii.planes)
    {
      freeobjects ();
      icon = ii;
      createobjects (hdc);
    }
  ReleaseDC (hwnd, hdc);
}

HICON
do_createicon (void)
{
  HICON hicon = NULL;

  if (icon.bitmap[0] && icon.bitmap[1])
    {
      BYTE buf[2][((icon.x * icon.bitspixel + 7) / 8) * icon.y * icon.planes];

      GetBitmapBits (icon.bitmap[0], sizeof buf[0], buf[0]);
      GetBitmapBits (icon.bitmap[1], sizeof buf[1], buf[1]);
      hicon = CreateIcon (hinst, icon.x, icon.y, icon.planes, icon.bitspixel,
			  buf[0], buf[1]);
    }
  return hicon;
}

/* x is cpu usage value between 0 (0.00%) and 10000 (100.00%) */
/* x=-1 for error */
void
next_data (HWND hwnd, int x)
{
  RECT rect;

  prepare (hwnd);

  if (x >= 0)
    {
      if (x > 10000)
	x = 10000;
      x = (10000 - x) * icon.y / 10000;
      if (x < 0)
	x = 0;
      if (x >= icon.y)
	x = icon.y - 1;
    }
  else
    {
      x = icon.y;
    }

  BitBlt (icon.dc[0], 0, 0, icon.x - 1, icon.y, icon.dc[0], 1, 0, SRCCOPY);
  BitBlt (icon.dc[1], 0, 0, icon.x - 1, icon.y, icon.dc[1], 1, 0, SRCCOPY);
  rect.left = icon.x - 1;
  rect.top = 0;
  rect.right = icon.x;
  rect.bottom = x;
  FillRect (icon.dc[0], &rect, icon.brush[0][1]);
  FillRect (icon.dc[1], &rect, icon.brush[1][1]);
  rect.top = x;
  rect.bottom = icon.y;
  FillRect (icon.dc[0], &rect, icon.brush[0][0]);
  FillRect (icon.dc[1], &rect, icon.brush[1][0]);
}
