/*
  Copyright (C) 2008  Hideki EIRAKU
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
*/

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include "icon.h"
#include "options.h"
#include "resource.h"
#include "reg.h"

struct options opt;
TCHAR regkey[] = TEXT ("Software\\hdk\\cpu%.exe");
TCHAR appname[] = TEXT ("cpu%.exe");

static int win6;

static void
loaddefault (void)
{
  opt.bgcolortype = 2;
  opt.bgcolor = RGB (0, 0, 0);
  opt.fgcolortype = 0;
  opt.fgcolor = RGB (0, 0, 0);
  if (win6)
    opt.fgcolor = RGB (0, 0, 255);
  opt.interval = 1000;
}

static void
save (void)
{
  openreg ();
  savesettingint (TEXT ("bgcolortype"), opt.bgcolortype);
  savesettingint (TEXT ("bgcolor"), opt.bgcolor);
  savesettingint (TEXT ("fgcolortype"), opt.fgcolortype);
  savesettingint (TEXT ("fgcolor"), opt.fgcolor);
  savesettingint (TEXT ("interval"), opt.interval);
  closereg ();
}

static int
colordialog (HWND hwnd, COLORREF *c)
{
  CHOOSECOLOR cc;
  static COLORREF cust[16];

  cc.lStructSize = sizeof cc;
  cc.hwndOwner = hwnd;
  cc.rgbResult = *c;
  cc.lpCustColors = cust;
  cc.Flags = CC_RGBINIT;
  if (ChooseColor (&cc))
    {
      *c = cc.rgbResult;
      return 1;
    }
  return 0;
}

static void
dlgsub (HWND hwnd)
{
  if (opt.bgcolortype & 2)
    SendDlgItemMessage (hwnd, IDC_CHECK1, BM_SETCHECK, BST_CHECKED, 0);
  else
    SendDlgItemMessage (hwnd, IDC_CHECK1, BM_SETCHECK, BST_UNCHECKED, 0);
  if (opt.fgcolortype & 2)
    SendDlgItemMessage (hwnd, IDC_CHECK2, BM_SETCHECK, BST_CHECKED, 0);
  else
    SendDlgItemMessage (hwnd, IDC_CHECK2, BM_SETCHECK, BST_UNCHECKED, 0);
  SendDlgItemMessage (hwnd, IDC_SLIDER1, TBM_SETPAGESIZE, 0, 10);
  SendDlgItemMessage (hwnd, IDC_SLIDER1, TBM_SETRANGE, TRUE,
		      MAKELPARAM (1, 20));
  SendDlgItemMessage (hwnd, IDC_SLIDER1, TBM_SETPOS, TRUE, opt.interval / 100);
}

static BOOL CALLBACK
dlgproc (HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
  int newval;

  if (msg == WM_INITDIALOG)
    {
      dlgsub (hwnd);
      return 1;
    }
  if (msg == WM_COMMAND)
    {
      switch (LOWORD (w))
	{
	case IDCANCEL:
	  EndDialog (hwnd, 0);
	  break;
	case IDC_CHECK1:
	  if (HIWORD (w) == BN_CLICKED)
	    {
	      if (Button_GetCheck ((HWND)l) == BST_CHECKED)
		opt.bgcolortype |= 2;
	      else
		opt.bgcolortype &= ~2;
	      icon_update ();
	      save ();
	    }
	  break;
	case IDC_CHECK2:
	  if (HIWORD (w) == BN_CLICKED)
	    {
	      if (Button_GetCheck ((HWND)l) == BST_CHECKED)
		opt.fgcolortype |= 2;
	      else
		opt.fgcolortype &= ~2;
	      icon_update ();
	      save ();
	    }
	  break;
	case IDC_BUTTON1:
	  if (HIWORD (w) == BN_CLICKED)
	    {
	      if (colordialog (hwnd, &opt.bgcolor))
		{
		  SendDlgItemMessage (hwnd, IDC_CHECK1, BM_SETCHECK,
				      BST_UNCHECKED, 0);
		  opt.bgcolortype = 1;
		  icon_update ();
		  save ();
		}
	    }
	  break;
	case IDC_BUTTON2:
	  if (HIWORD (w) == BN_CLICKED)
	    {
	      if (colordialog (hwnd, &opt.fgcolor))
		{
		  SendDlgItemMessage (hwnd, IDC_CHECK2, BM_SETCHECK,
				      BST_UNCHECKED, 0);
		  opt.fgcolortype = 1;
		  icon_update ();
		  save ();
		}
	    }
	  break;
	case IDC_BUTTON3:
	  if (HIWORD (w) == BN_CLICKED)
	    {
	      loaddefault ();
	      dlgsub (hwnd);
	      SetTimer (GetParent (hwnd), 1, opt.interval, NULL);
	      icon_update ();
	      save ();
	    }
	  break;
	}
      return 1;
    }
  if (msg == WM_HSCROLL)
    {
      newval = SendDlgItemMessage (hwnd, IDC_SLIDER1, TBM_GETPOS, 0, 0) * 100;
      if (newval <= 0)
	newval = 1;
      if (newval != opt.interval)
	{
	  opt.interval = newval;
	  SetTimer (GetParent (hwnd), 1, opt.interval, NULL);
	  save ();
	}
      return 1;
    }
  return 0;
}

void
optionsdlg (HWND hwnd)
{
  DialogBox (GetModuleHandle (0), MAKEINTRESOURCE (IDD_DIALOG1), hwnd,
	     dlgproc);
}

void
loadoptions (int vistaflag)
{
  win6 = vistaflag;
  loaddefault ();
  openreg ();
  opt.bgcolortype = getsettingint (TEXT ("bgcolortype"), opt.bgcolortype);
  if (opt.bgcolortype & 1)
    opt.bgcolor = getsettingint (TEXT ("bgcolor"), opt.bgcolor);
  opt.fgcolortype = getsettingint (TEXT ("fgcolortype"), opt.fgcolortype);
  if (opt.fgcolortype & 1)
    opt.fgcolor = getsettingint (TEXT ("fgcolor"), opt.fgcolor);
  opt.interval = getsettingint (TEXT ("interval"), opt.interval);
  closereg ();
}
