/*
  Copyright (C) 2008  Hideki EIRAKU
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
*/

#include <windows.h>
#include "reg.h"

static int regcount;
static HKEY hkey;
static int usereg;
static int initflag;
static const int settinglen = 512;

#define fatalerror() \
	fatalerrorfunc (TEXT (__FILE__), __LINE__)

static void
fatalerrorfunc (TCHAR *file, int line)
{
  TCHAR buf[1024];

  wsprintf (buf, TEXT ("FATAL ERROR file %s line %d"), file, line);
  MessageBox (NULL, buf, 0,
	      MB_TASKMODAL | MB_ICONEXCLAMATION | MB_OK);
  ExitProcess (1);
}

void
openreg (void)
{
  if (!initflag)
    {
      initflag = 1;
      hkey = NULL;
      usereg = 1;
      openreg ();
      if (!hkey)
	{
	  usereg = 0;
	  usereg = getsettingint (TEXT ("UseRegistry"), 1);
	}
      closereg ();
    }
  regcount++;
  if (!hkey && usereg)
    if (RegOpenKeyEx (HKEY_CURRENT_USER, regkey, 0,
		      KEY_READ | KEY_WRITE, &hkey) != ERROR_SUCCESS)
      hkey = 0;
}

void
closereg (void)
{
  regcount--;
  if (hkey && regcount == 0)
    {
      if (RegCloseKey (hkey) != ERROR_SUCCESS)
	fatalerror ();
      else
	hkey = 0;
    }
  if (!usereg)
    {
      TCHAR modname[300];
      TCHAR *p;

      if (!GetModuleFileName (0, modname, 300))
	fatalerror ();
      p = modname;
      p += lstrlen (modname);
      while (p != modname && *p != TEXT ('.'))
	p = CharPrev (modname, p);
      if (!lstrcpy (p, TEXT (".ini")))
	fatalerror ();
      WritePrivateProfileString (0, 0, 0, modname);
    }
}

int
getsettingint (const TCHAR *keyname, int def)
{
  int r;
  DWORD type, size;
  TCHAR key[lstrlen (keyname) + 7];

  if (regcount <= 0)
    fatalerror ();
  lstrcpy (key, TEXT ("DWORD_"));
  lstrcat (key, keyname);
  if (!hkey)
    {
      TCHAR modname[300];
      TCHAR *p;

      if (!GetModuleFileName (0, modname, 300))
	return def;
      p = modname;
      p += lstrlen (modname);
      while (p != modname && *p != TEXT ('.'))
	p = CharPrev (modname, p);
      if (!lstrcpy (p, TEXT (".ini")))
	return def;
      if (usereg)
	ini2reg ();
      return (int)GetPrivateProfileInt (appname, key, def, modname);
    }
  type = REG_DWORD;
  size = sizeof (r);
  if (RegQueryValueEx (hkey, key, 0, &type, (BYTE *)&r, &size)
      != ERROR_SUCCESS)
    r = def;
  return r;
}

void
getsettingstr (const TCHAR *keyname, const TCHAR *def, TCHAR *buf)
{
  DWORD type, size;
  TCHAR key[lstrlen (keyname) + 4];

  if (regcount <= 0)
    fatalerror ();
  lstrcpy (key, TEXT ("SZ_"));
  lstrcat (key, keyname);
  if (!hkey)
    {
      TCHAR modname[300];
      TCHAR *p;

      if (!GetModuleFileName (0, modname, 300))
	{
	  lstrcpyn (buf, def, settinglen);
	  return;
	}
      p = modname;
      p += lstrlen (modname);
      while (p != modname && *p != TEXT ('.'))
	p = CharPrev (modname, p);
      if (!lstrcpy (p, TEXT (".ini")))
	{
	  lstrcpyn (buf, def, settinglen);
	  return;
	}
      if (usereg)
	ini2reg ();
      GetPrivateProfileString (appname, key, def, buf,
			       (DWORD)settinglen, modname);
      return;
    }
  type = REG_SZ;
  size = (DWORD)(sizeof (TCHAR) * settinglen);
  if (RegQueryValueEx (hkey, key, 0, &type, (BYTE *)buf, &size)
      != ERROR_SUCCESS)
    lstrcpyn (buf, def, settinglen);
}

void
savesettingstr (const TCHAR *keyname, const TCHAR *buf)
{
  DWORD type, size;
  TCHAR key[lstrlen (keyname) + 4];

  if (regcount <= 0)
    fatalerror ();
  lstrcpy (key, TEXT ("SZ_"));
  lstrcat (key, keyname);
  if (!hkey)
    {
      TCHAR modname[300];
      TCHAR *p;

      if (!GetModuleFileName (0, modname, 300))
	fatalerror ();
      p = modname;
      p += lstrlen (modname);
      while (p != modname && *p != TEXT ('.'))
	p = CharPrev (modname, p);
      if (!lstrcpy (p, TEXT (".ini")))
	fatalerror ();
      if (WritePrivateProfileString (appname, key, buf, modname))
	return;
      ini2reg ();
      usereg = 1;
      closereg ();
      openreg ();
      if (!hkey)
	fatalerror ();
    }
  type = REG_SZ;
  size = sizeof (TCHAR) * (lstrlen (buf) + 1);
  if (RegSetValueEx (hkey, key, 0, type, (BYTE *)buf, size)
      != ERROR_SUCCESS)
    fatalerror ();
}

void
savesettingint (const TCHAR *keyname, int value)
{
  DWORD type, size;
  TCHAR key[lstrlen (keyname) + 7];

  if (regcount <= 0)
    fatalerror ();
  lstrcpy (key, TEXT ("DWORD_"));
  lstrcat (key, keyname);
  if (!hkey)
    {
      TCHAR modname[300];
      TCHAR *p;
      TCHAR buf[16];

      if (!GetModuleFileName (0, modname, 300))
	fatalerror ();
      p = modname;
      p += lstrlen (modname);
      while (p != modname && *p != TEXT ('.'))
	p = CharPrev (modname, p);
      if (!lstrcpy (p, TEXT (".ini")))
	fatalerror ();
      wsprintf (buf, TEXT ("%d"), value);
      if (WritePrivateProfileString (appname, key, buf, modname))
	return;
      ini2reg ();
      usereg = 1;
      closereg ();
      openreg ();
      if (!hkey)
	fatalerror ();
    }
  type = REG_DWORD;
  size = sizeof (value);
  if (RegSetValueEx (hkey, key, 0, type, (BYTE *)&value, size)
      != ERROR_SUCCESS)
    fatalerror ();
}

void
ini2reg (void)
{
  DWORD tmp;
  TCHAR *buf;
  TCHAR modname[300];
  TCHAR *p, *q;
  TCHAR buf2[settinglen];
  UINT tmp2;

  if (hkey)
    fatalerror ();
  if (!GetModuleFileName (0, modname, 300))
    fatalerror ();
  p = modname;
  p += lstrlen (modname);
  while (p != modname && *p != TEXT ('.'))
    p = CharPrev (modname, p);
  if (!lstrcpy (p, TEXT (".ini")))
    fatalerror ();
  buf = (TCHAR *)LocalAlloc (LMEM_FIXED, sizeof (TCHAR) * 32768);
  if (!buf)
    fatalerror ();
  GetPrivateProfileSection (appname, buf, 32767, modname);
  if (RegCreateKeyEx (HKEY_CURRENT_USER, regkey, 0,
		      0, REG_OPTION_NON_VOLATILE, KEY_READ | KEY_WRITE, 0,
		      &hkey, &tmp)
      != ERROR_SUCCESS)
    fatalerror ();
  if (RegCloseKey (hkey) != ERROR_SUCCESS)
    fatalerror ();
  hkey = 0;
  usereg = 1;
  openreg ();
  p = buf;
  while (*p)
    {
      q = p;
      while (*q != TEXT ('=') && *q)
	q++;
      if (*q == 0)
	break;
      *q = 0;
      if (*p == TEXT ('D'))
	{
	  tmp2 = GetPrivateProfileInt (appname, p, 0, modname);
	  savesettingint (&p[6], (int)tmp2);
	}
      else
	{
	  GetPrivateProfileString (appname, p, TEXT (""), buf2,
				   (DWORD)settinglen, modname);
	  savesettingstr (&p[3], buf2);
	}
      *q = TEXT ('=');
      p = q;
      while (*p)
	p++;
      p++;
    }
  closereg ();
  if (LocalFree (buf))
    fatalerror ();
}
